Q

Struct Q 

pub struct Q<T, A, const F: i8> { /* private fields */ }
Fixed point integer

Generics:

  • T: Base integer
  • A: Accumulator for intermediate results
  • F: Number of fractional bits right of the decimal point

F negative is supported analogously.

  • Q32<31> is (-1..1).step_by(2^-31)
  • Q<i16, _, 20> is (-1/32..1/32).step_by(2^-20)
  • Q<u8, _, 4> is (0..16).step_by(1/16)
  • Q<u8, _, -2> is (0..1024).step_by(4)
assert_eq!(Q8::<4>::from_int(3), Q8::from_bits(3 << 4));
assert_eq!(7 * Q8::<4>::from_f32(1.5), 10);
assert_eq!(Q8::<4>::from_f32(1.5).apply(7), 10);
assert_eq!(7 / Q8::<4>::from_f32(1.5), 4);

Implementations

Source

impl<T, A, const F: i8> Q<T, A, F>

pub const DELTA: f32

Step between distinct numbers

assert_eq!(Q32::<31>::DELTA, 2f32.powi(-31));
assert_eq!(Q32::<-4>::DELTA, 2f32.powi(4));
let _ = Q32::<-128>::DELTA;
pub const fn from_bits(bits: T) -> Self

Create a new fixed point number from a raw representation.

pub fn into_bits(self) -> T

Return the raw representation.

Source

impl<T: Shift, A, const F: i8> Q<T, A, F>

pub fn scale<const F1: i8>(self) -> Q<T, A, F1>

Convert to a different number of fractional bits (truncating)

Use this liberally for Add/Sub/Rem with Q’s of different F.

assert_eq!(Q8::<4>::from_bits(32).scale::<0>(), Q8::from_bits(2));
pub fn trunc(self) -> T

Return the integer part

assert_eq!(Q8::<4>::from_bits(0x35).trunc(), 0x3);
pub fn from_int(value: T) -> Self

Scale from integer base type

assert_eq!(Q8::<4>::from_int(7).into_bits(), 7 << 4);
Source

impl<A: Shift, T: Accu<A>, const F: i8> Q<A, T, F>

pub fn quantize(self) -> T

Scale from integer accu type

let q = Q8::<4>::from_f32(0.25);
assert_eq!((q * 7).quantize(), (7.0 * 0.25f32).floor() as _);
Source

impl<T: Accu<A>, A: Mul<Output = A>, const F: i8> Q<T, A, F>

pub fn mul_wide(self, rhs: T) -> Q<A, T, F>

Multiply by a raw integer and keep the widened accumulator result.

assert_eq!(Q8::<3>::from_bits(4).mul_wide(2), Q::from_bits(8));
Source

impl<T: Accu<A>, A: Shift + Mul<Output = A>, const F: i8> Q<T, A, F>

pub fn apply(self, rhs: T) -> T

Apply this fixed-point value as a gain to a raw integer and quantize.

assert_eq!(Q8::<4>::from_f32(0.25).apply(7), 1);
Source

impl<T, A, const F: i8> Q<T, A, F>
where f32: AsPrimitive<Q<T, A, F>>, Self: Copy + 'static,

pub fn from_f32(value: f32) -> Self

Quantize a f32

Source

impl<T, A, const F: i8> Q<T, A, F>
where f64: AsPrimitive<Q<T, A, F>>, Self: Copy + 'static,

pub fn from_f64(value: f64) -> Self

Quantize a f64

Source

impl<T: AsFloat, A, const F: i8> Q<T, A, F>

pub fn as_f32(self) -> f32

Convert lossy to f32

pub fn as_f64(self) -> f64

Convert lossy to f64