Module serde

Module serde 

Serde adapters for fixed-point wire formats.

The direct Q serde implementation is transparent and uses the raw representation. Use as_f32 or as_f64 when the wire format should carry the scaled value through serde’s floating-point data model instead.

Modules

as_f32
Serialize and lossy-deserialize Q through serde’s f32 data model.
as_f64
Serialize and lossy-deserialize Q through serde’s f64 data model.